;    This file is part of hb, halfband-led-spectrumanalyzer
;
;    hb is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    hb is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with hb.  If not, see <http://www.gnu.org/licenses/>.

;-----------------------------------------------------------------------------
.include "fstruct.inc"
;-----------------------------------------------------------------------------
; expects input samples in Y[0..x]
; returns output sample in r18
; modifies: r1, r0,  : product
;           r11, r10  : lowpass result 
;           r13, r12  : highpass result
;           r15, r14  : sum of products in filterkernel
;                r16  : sum of squares of hi pass
;           r19, r18  : coeff, sample
;           r29, r28  : Y

; old filter coeff:
; 4,0,-10,0,23,0,-80,-128,-80,0,23,0,-10,0,4
;
;alternative coeff - provides zero at band end:
; gnu octave: round(-256*firls(14,[0 0.38 0.62 1],[1 1 0 0]))
; 3,0,-9,0,22,0,-80,-128,-80,0,22,0,-9,0,3

; Memory Layout of filter  struct:
;   15 byte samples   <-y points to the 1st sample at start of filtering
;   y+0...y+14 : FIR chain
;
;   3 byte sum of high pass squares
;   y+15(LSB)...y+17(MSB) : Sum of hipass squares

;  0 byte low pass sample xferred already to the next filter struct
.globl fir
fir:
    ldi r18, 3
    ldd r19, Y+0
    mulsu r19,r18
    movw r14,r0
    ldd r19, Y+14
    mulsu r19,r18
    add r14,r0
    adc r15,r1

    ldi r18, 9
    ldd r19, Y+2
    mulsu r19,r18
    sub r14,r0
    sbc r15,r1
    ldd r19, Y+12
    mulsu r19,r18
    sub r14,r0
    sbc r15,r1

    ldi r18, 22
    ldd r19, Y+4
    mulsu r19,r18
    add r14,r0
    adc r15,r1
    ldd r19, Y+10
    mulsu r19,r18
    add r14,r0
    adc r15,r1

    ldi r18, 80
    ldd r19, Y+6
    mulsu r19,r18
    sub r14,r0
    sbc r15,r1
    ldd r19, Y+8
    mulsu r19,r18
    sub r14,r0
    sbc r15,r1

    movw r10,r14
    movw r12,r14

    ldi r18, 128
    ldd r19, Y+7
    mulsu r19,r18

    sub r10,r0
    sbc r11,r1

    add r12,r0
    adc r13,r1
; lp+hp result ready : r10/11 -> LowPass, r12/13 Highpass
; square high pass MSByte and add to sum (0 <=r1 <=64)
    mov r18,r13
    muls r18,r18
    ldd r14,Y+sqso+0
    ldd r15,Y+sqso+1
    ldd r16,Y+sqso+2
    add r14,r0
    adc r15,r1
    brcc no16bitcy
    inc r16 ; holy crap: there's no such opcode like "adc r16,#0"
no16bitcy:
    std Y+sqso+0,r14
    std Y+sqso+1,r15
    std Y+sqso+2,r16
; lowpass result in r18, where it's needed for the next filter
    mov  r18,r11
; now blockmove 15-2=13 byte samples by 2 byte 
;  obacht: change here if # of fir taps changes
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    ldd r14,Y+2;
    st Y+,r14;
    adiw r28, fsize-13 
    ;  Y shouldpoint now to the next filter struct
    ret
;-----------------------------------------------------------------------------
.globl sample2bardot
; expects 9bit input sample in carry + r16
; expects y to point to actual filterstruct
; modifies r16, r17
; modifies y[bdso],y[bdco]

sample2bardot:
    ldi r17,0x09 ; calculate rough estimation of "logarithmus dualis"
    brcs logdualisdone
logdualisloop:
    dec r17
    breq logdualisdone
    lsl r16
    brcc logdualisloop
logdualisdone:
    ldd r16,y+bdso
    andi r16,0xf0
    or r17,r16
    std y+bdso,r17 ; new bar/dot status written - jep tatsaechlich.danach scheints nie mehr
    mov r16,r17
    swap r16
    andi r16,0x0f  ; dot in LSN of r16
    andi r17,0x0f  ; bar in LSN of r17
    cp r17,r16     
    brcs no_bar_to_dot_copy
    ;bar is same or bigger than dot, so cp bar->dot and reset counter...
    mov r16,r17
    swap r16 ; dot in MSN of r16
    or r17,r16
    std Y+bdso,r17 ; new bar/dot status written
    eor r16,r16
    std Y+bdco,r16;  dotcounter=0
    ret
no_bar_to_dot_copy:
    ldd r16,Y+bdco; load dotcounter
    ldd r17,Y+bdso; load bar/dot status
    cpi r16,0xff
    brne no_counter_done
    ; dotcounter has reached 255, so be sure to set dot=0
    andi r17,0x0f
    std Y+bdso,r17 ; new bar/dot status written
    ret
no_counter_done:
    inc r16 ;  dotcounter++
    std Y+bdco,r16;
    cpi r16,0x07 ; time to wait until dots count down
    brcc downcount_dots
    ret
downcount_dots:
    andi r16,0x00 ;1,3,7 : speed to count
    breq maybe_downcount
    ret
maybe_downcount:
    mov r16,r17
    andi r16,0xf0
    brne really_downcount
    ret ; dots already counted to 0
really_downcount:
    subi r17,0x10
    std Y+bdso,r17 ; new bar/dot status written
    ret

