;    This file is part of hb, halfband-led-spectrumanalyzer
;
;    hb is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    hb is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with hb.  If not, see <http://www.gnu.org/licenses/>.

;-----------------------------------------------------------------------------
.include "fstruct.inc"
;-----------------------------------------------------------------------------
; main gedoense
; DWTF

; r2,r3 : toggle bits
; r31:r30 : z-register; used as write ptr for the samples from ADC to fifo
; r29:r28 : y-register; used as ptr to different filter structs
; r27:r26 : x-register; used ad read ptr for the samples from fifo to filters

; 256 bytes adc fifo buffer : 0x100 .. 0x1ff
; start of filterstructs @ address 0x200
;


;-------------------------------------------------------------
; toggle_bit : perform decimation, that's: call the filter every 2nd time only
.macro toggle_bit bitno
    bst r2,\bitno
    brts 1f
    set
    bld r2,\bitno
    std y+13,r18
    jmp main_loop
1:
    clt
    bld r2,\bitno
    std y+14,r18
.endm
;-------------------------------------------------------------
; initialize things
.globl main
main:
    ldi r28,0x5f
    ldi r29,0x04
    out 0x3e,r29
    out 0x3d,r28; stackpointer to end of RAM

    in r28, 0x34
    ori r28,0x80
    out 0x34,r28
    out 0x34,r28; enable port C lines; disable JTAG

    ldi r28,0xff
    out 0x11,r28; ddrd
    out 0x14,r28; ddrc
    out 0x15,r28; portc : all Display LEDs off
    out 0x12,r28; portd : all columns off; CPU-load led on

    ldi r29,0x01
    ldi r28,0xf3
    out 0x2b,r29
    out 0x2a,r28; ocr1a = 499   ->   16MHz / 500 = 32kHz ADC sampling frequency

    ldi r28,0x00
    out 0x1f,r28; EEARH=0, the bar/dot lookuptable is just 256 bytes
    out 0x2f,r28; tccr1a = 0
    ldi r28,0x09
    out 0x2e,r28; tccr1b = 9
    ldi r28,0xed
    out 0x06,r28; adcsra=0xed
    ldi r28,0xe0
    out 0x07,r28; admux=0xe0

    in r28,0x30
    andi r28,0x0f
    ori  r28,0xa0
    out  0x30,r28; sfior upper nibble =0xa

    ;we use ram from 0x100..0x1ff as 256 bytes fifo for ADC samples
    ldi r27,0x01
    ldi r26,0x00;  x=0x100 -> start of 256 bytes buffer; rd-ptr
    ldi r31,0x01
    ldi r30,0x00; z=0x100 -> start of 256 bytes buffer; wr-ptr

    eor r4,r4 ; initialize column counter


    sei; now things get serious...


;----------------------------------------------------------------------
main_loop:
    cp r26,r30 ; compare rd & wr pointer
    brne new_sample
    ; do now things while waiting for new samples...

    lds r28,0x60
    lds r29,0x61
    adiw r28,1
    sts 0x60,r28
    sts 0x61,r29
    lds 28,0x62
    sbci r28,0
    sts 0x62,r28
    bst r28,1

    cli ; obacht - no irqs during changes on the LED/column driver port!
        in r28,0x12
        bld r28,6
        out 0x12,r28 ; make CPU-load led blink
    sei ; now it's safe again to permit irqs
    rjmp main_loop

;-----------------------------------------------------------------------

; if we're here, there's at least one byte of audiosamples available
new_sample:
    ld r18,x
    inc r26      ; rd-ptr++
    ldi r28,0x00
    ldi r29,0x02 ; y points to 1st filter struct (0x200)

toggle_bit 0
fir0:  ; we get here on every 2nd sample from the adc -> 16000x per sec
    call fir 
    toggle_bit 1
fir1: ; 8k/sec
    call fir 
    toggle_bit 2
fir2: ; 4k/sec
    call fir 
    toggle_bit 3
fir3: ; 2k/sec
    call fir;
    toggle_bit 4
fir4: ; 1k/sec
    call fir; 
    toggle_bit 5
fir_5: ; 500/sec
    call fir; 
    toggle_bit 6
fir6: ; 250/sec 
    call fir; 
    toggle_bit 7
fir7: ; 125/sec
    call fir; 
; toggle_bit 8: not as a macro...
    bst r3,0
    brts tb8
    set
    bld r3,0
    std y+13,r18
    jmp main_loop
tb8:
    clt
    bld r3,0
    std y+14,r18

;---end toggle bit 8
fir8: ; 62.5/sec
    call fir
; toggle_bit 9: not as a macro...
    bst r3,1
    brts tb9
    set
    bld r3,1
    std y+13,r18
    jmp main_loop
tb9:
    clt
    bld r3,1
    std y+14,r18

;---end toggle bit 9
fir9: ; 31.5/sec
    call fir


;now calc. log. from sum of squares of hi pass :
    ldi r28,0x00
    ldi r29,0x02 ; y points to filterstructs
    eor r20,r20 ; outer loopcounter=0
loop1:
    ldd  r14,y+sqso
    ldd  r15,y+sqso+1
    ldd  r16,y+sqso+2
    mov  r9,r20
    inc  r9 ; put more inc here to be more sensitive
    inc  r9 ; put more inc here to be more sensitive
    inc  r9 ; put more inc here to be more sensitive
    inc  r9 ; put more inc here to be more sensitive
    inc  r9 ; put more inc here to be more sensitive
loop2:
    lsl  r14
    rol  r15
    rol  r16
    dec  r9
    brne loop2
    call sample2bardot
    eor  r16,r16 ; zero out sum of squares
    std  y+sqso,r16
    std  y+sqso+1,r16
    std  y+sqso+2,r16
    adiw r28,fsize
    inc r20
    cpi r20,0xa ; 10 columns
    brne loop1
    jmp main_loop
